# Converts an AGI file, WORDS.TOK to an SCI file, VOCAB.000
# Some things are beyond the scope of this program, such as assigning parts
# of speech to each word. The word group number is used to determine the
# part of speech code that should be used: 0 for noword, not 0 for anyword.
# Do whatever you want with this file, except removing my name.
# John Nickerson, mokalus@yahoo.com.au 31jan2002
# version 1.1.0b

def setclasscode(w):
	"Returns an SCI Vocab class code from an AGI word group number."
	if w == 0:
		return chr(0)+chr(0)+chr(0)
	else:
		if (((w/256)+240)>255):
			print "\nWord number out of range: " + str(w)
			print "TOK2VOC can only handle word numbers from 0 to 4095."
			print "This is an SCI limitation, and cannot be fixed."
			print "Edit the source file, and try again."
			agi = raw_input("Press ENTER to finish.")
			sys.exit(1)
		else:
			return chr(255) + chr((w / 256) + 240) + chr(w % 256)

import sys
alphaindex = [55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] # To store the file header
renumber = {} # Stores the word numbers as reassigned by TOK2VOC
freenumbers = range(1,4096) # Stores unused numbers

if len(sys.argv) < 2:
	filename = raw_input("AGI WORDS.TOK input file location:")
else:
	filename = sys.argv[1]
if filename[-1] != "\\":
	filename = filename + "\\words.tok"
else:
	filename = filename + "words.tok"
try:
	wordstok = open(filename,"rb")
except EnvironmentError:
	print "File not found:" + filename
	print "Check that the file exists, and that the name is correct."
	agi = raw_input("Press ENTER to finish.")
	sys.exit(1)
if len(sys.argv) < 3:
	filename = raw_input("SCI VOCAB.000 output file location:")
else:
	filename = sys.argv[2]
if filename[-1] != "\\":
	filename = filename + "\\vocab.000"
else:
	filename = filename + "vocab.000"
try:
	vocab000 = open(filename,"wb")
except EnvironmentError:
	print "File not found:" + filename
	print "Check that the file exists, and that the name is correct."
	agi = raw_input("Press ENTER to finish.")
	sys.exit(1)
del filename
vocab000.write("\x86\x00") # SCI file marker
writebuf = ""

print "AGI to SCI vocabulary converter.\nJohn Nickerson. January 2002\n"

print "Beginning transformation..."
y = 55 # Imaginary position in the VOCAB.000 file - to construct the header
wordstok.seek(52) # Skip over the entire WORDS.TOK header - it's useless to us.
agi = wordstok.read(1)
sci = agi
writebuf += sci # First prefix
x = 53
while 1:
	wordstok.seek(x)
	agi = wordstok.read(1)
	if agi == '':
		break # End Of File encountered.
	if ord(agi) > 127: # High bit is set - end of word.
		# Convert to SCI format
		sci = chr(127 ^ ord(agi))
		y = y+1
		writebuf += sci
#		print sci,
		# Determine the word's group number.
		agi = wordstok.read(2)
		wordnum = (ord(agi[0]) * 256) + ord(agi[1])
		if wordnum == 0:
			classcode = setclasscode(wordnum)
		elif wordnum in renumber.keys():
			classcode = setclasscode(renumber[wordnum])
		elif wordnum in freenumbers:
			renumber[wordnum] = wordnum
			freenumbers = filter(lambda x: x != wordnum, freenumbers)
			classcode = setclasscode(renumber[wordnum])
		else:
			if len(freenumbers) == 0:
				print "Out of word numbers.\n"
				print "TOK2VOC attempts to squeeze all word numbers from a potential range"
				print "of 0 to 65535 into a range of 0 to 4095, because of a limitation of"
				print "the SCI engine. If more word numbers are used than will fit, TOK2VOC"
				print "can't cope.\n"
				print "Try scaling down your game efforts a bit. I'm sure you can manage with"
				print "a smaller vocabulary."
				agi = raw_input("Press ENTER to finish.")
				sys.exit(1)
			renumber[wordnum] = freenumbers[0]
			freenumbers = freenumbers[1:]
			classcode = setclasscode(renumber[wordnum])
		y = y+1
		writebuf += classcode
		# Get the next prefix.
		agi = wordstok.read(1)
		sci = agi
		y = y+3
		writebuf += sci
		if ord(agi) == 0:
			agi = wordstok.read(1)
			if len(agi) > 0:
				b = (127 & (ord(agi) ^ 127))
				if ord("a") <= b <= ord("z"):
					alphaindex[b - ord("a")] = y
					print chr(b),
#					sci = raw_input("")
		x = x+3
	else:
		# An ordinary mid-word character, encrypted.
		sci = chr(ord(agi) ^ 127)
#		print sci,
		y = y+1
		writebuf += sci
	x = x+1
print
print alphaindex
# Write out the header.
print "Writing index..."
for x in range(0,26):
	# alphaindex[x] = alphaindex[x]-1
	y = x*2
	sci = chr(alphaindex[x] / 256)
	vocab000.write(sci)
	sci = chr(alphaindex[x] % 256)
	vocab000.write(sci)
	print chr(ord("a") + x),
print
print "Writing words..."
while len(writebuf) > 0:
	vocab000.write(writebuf[0:100])
	writebuf = writebuf[100:]
	print "#",
print "\nDone."
if len(sys.argv) < 2:
	printbuf = raw_input('Press ENTER key to exit.')
del agi,alphaindex,b,classcode,freenumbers,printbuf,renumber,sci
del vocab000,wordnum,wordstok,writebuf,x,y
